###################################################################################
# Ruby-script to export the content of a KSpread sheet into a CSV file.
#
# Description:   This script exports data from KSpread to a HTML file.
# Author:        Sebastian Sauer <mail@dipe.org>
# Copyright:     Dual-licensed under LGPL v2+higher and the BSD license.
###################################################################################

require 'Qt'
require 'csv'

class Dialog < Qt::Dialog
	slots 'export()'
	slots 'browse()'

	def initialize()
		super(nil,nil,true,Qt::WDestructiveClose)
		setCaption('Export to Text File')

		require "krosskspreadcore"
		@document = Krosskspreadcore::get("KSpreadDocument")

		layout = Qt::VBoxLayout.new(self)
		box = Qt::VBox.new(self)
		box.setMargin(10)
		box.setSpacing(10)
		layout.addWidget(box)

		sheetbox = Qt::HBox.new(box)
		sheetbox.setSpacing(10)
		Qt::Label.new('Sheet:', sheetbox)
		@sheetcombo = Qt::ComboBox.new(sheetbox)
		sheetbox.setStretchFactor(@sheetcombo,1)
		for sheetname in @document.sheetNames()
			@sheetcombo.insertItem(sheetname)
		end

		filebox = Qt::HBox.new(box)
		filebox.setSpacing(10)
		Qt::Label.new('File:', filebox)
		@filedit = Qt::LineEdit.new(filebox)
		btn = Qt::PushButton.new('...', filebox)
		Qt::Object.connect(btn, SIGNAL('clicked()'), self, SLOT('browse()'))

		btnbox = Qt::HBox.new(box)
		btnbox.setSpacing(10)
		okbtn = Qt::PushButton.new('Export', btnbox)
		Qt::Object.connect(okbtn, SIGNAL('clicked()'), self, SLOT('export()'))
		quitbtn = Qt::PushButton.new('Cancel', btnbox)
		Qt::Object.connect(quitbtn, SIGNAL('clicked()'), self, SLOT('close()'))

		box.setMinimumSize( box.sizeHint() )
		box.setMinimumWidth(440)
	end

	def browse()
		filename = Qt::FileDialog.getSaveFileName(@filedit.text(),"*.csv *.txt;;*", self)
		if filename != nil
			@filedit.setText(filename)
		end
	end

	def export()
		sheetname = @sheetcombo.currentText()
		sheet = @document.sheetByName( sheetname )
		if not sheet
			Qt::MessageBox.critical(self,"Error","<qt>No so sheet \"" + sheetname + "\".</qt>")
			return
		end

		begin
			file = @filedit.text()
			outfile = File.open(file, 'wb')
			CSV::Writer.generate(outfile) do |csv|
				record = []
				cell = sheet.firstCell()
				while cell
					row = cell.row()
					record << cell.value()
					cell = cell.nextCell()
					if ( ( not cell ) or ( cell.row() != row ) )
						csv << record
						record = []
					end
				end
			end
			outfile.close
			self.close()
		rescue
			Qt::MessageBox.critical(self,"Error","<qt>Failed to write file \"" + file + "\"<br><br>" + $! + "</qt>")
		end
	end

end

dialog = Dialog.new()
dialog.exec()
